import java.util.Scanner;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;

class Reisehjelpen {
  public static void main (String[] args) throws Exception {
    Katalog destKatalog = new Katalog ("Katalog.txt");
    destKatalog.nyDestFraFil ("Stockholm", "Stockholm.txt");
    destKatalog.nyDestFraFil ("Paris", "Paris.txt");
    destKatalog.skrivDestListe ();

    destKatalog.skrivEnDest("Paris");
    destKatalog.skrivEnDest("Oslo");
    destKatalog.skrivEnDest("Stockholm");
    destKatalog.skrivEnDest("Berlin");
    destKatalog.skrivEnDest("London");

    System.out.println ("Beste destinasjon: ");
    System.out.println (destKatalog.finnBesteDest(true, 151220, 160105));
  }
}

class Attraksjon {
  String aNavn;
  int apnes, stenges;
  boolean bVennlig;

  public Attraksjon (String navn,boolean barn, int fra, int til) {
    aNavn = navn;
    apnes = fra;
    bVennlig = barn;
    stenges = til;
  }

  public void skrivAttr () {
    String utskrift = (aNavn + ". " + apnes + "-" + stenges);
    if (bVennlig) {
      utskrift = "BARN: " + utskrift;
    } else {
      utskrift = "VOKSNE: " + utskrift;
    }
    System.out.println (utskrift);
  }

  public boolean forBarn () {
    return bVennlig;
  }

  public boolean aapenIPeriode (int fra, int til) {
    if ((fra > stenges) || (til < apnes)) {
      return false;
    } else {
      return true;
    }
  }
}

class Destinasjon {
  private String dNavn;
  private ArrayList<Attraksjon> lokaleAttr;

  public Destinasjon (String navn, ArrayList<Attraksjon> attrListe) {
    dNavn = navn;
    lokaleAttr = attrListe;
  }

  public String hentDestNavn () {
    return dNavn;
  }

  public void skrivDest () {
    System.out.println ("Destinasjon " + dNavn);
    for (Attraksjon a : lokaleAttr) {
      a.skrivAttr();
    }
  }


  public void leggTilAttr (Attraksjon nyAttr) {
    lokaleAttr.add(nyAttr);
  }

  public int antallAktuelleAttr (boolean barn, int fra, int til){
    int antall = 0;
    for (Attraksjon a : lokaleAttr) {
      if ((barn && a.forBarn()) || (!barn)) {
        if (a.aapenIPeriode(fra, til)) {
          antall++;
        }
      }
    }
    return antall;
  }
}

class Katalog {
  private HashMap<String, Destinasjon> destKatalog =
                                       new HashMap<>();

  public Katalog (String katalogfil) throws Exception {
    lesDestinasjonsfil(katalogfil);
  }

  private void lesDestinasjonsfil (String filnavn) throws Exception{
    File fil = new File(filnavn);
    Scanner destFil = new Scanner(fil);
    while (destFil.hasNextLine()) {
      String nvn = destFil.nextLine();
      ArrayList<Attraksjon> attr = new ArrayList<>();
      String lest = destFil.nextLine();
      while (!lest.equals("*")) {
        String attrNavn = lest;
        String voksnebarn = destFil.nextLine();
        boolean barn = (voksnebarn.equals("BARN"));
        int fra = Integer.parseInt (destFil.nextLine());
        int til = Integer.parseInt (destFil.nextLine());
        Attraksjon nyAttr = new Attraksjon (attrNavn, barn, fra, til);
        attr.add (nyAttr);
        lest = destFil.nextLine();
      }
      Destinasjon nyDest = new Destinasjon (nvn, attr);
      destKatalog.put(nvn, nyDest);
    }
  }


  public void skrivDestListe () {
    System.out.println ("Destinasjoner i katalogen:");
    for (Destinasjon d : destKatalog.values()) {
      System.out.println (d.hentDestNavn());
    }
  }

  public void skrivEnDest (String dNavn) {
    Destinasjon dest = destKatalog.get(dNavn);
    if (dest != null) {
      dest.skrivDest();
    } else {
      System.out.println ("Ingen destinasjoner med navnet " + dNavn);
    }
  }

  public void nyAttr (String destNavn, String attrNavn,
  boolean bVennlig, int apenFra, int apenTil) {
    Destinasjon dest = destKatalog.get (destNavn);
    if (dest != null) {
      Attraksjon nyAttr = new Attraksjon (attrNavn, bVennlig,
      apenFra, apenTil);
      dest.leggTilAttr(nyAttr);
    }
  }

  public void nyDestFraFil (String destNavn, String filnavn)
  throws Exception {
    if (destKatalog.get(destNavn) == null) {
      Destinasjon d = new Destinasjon (destNavn,
      new ArrayList<>());
      destKatalog.put (destNavn, d);
    }

    File fil = new File(filnavn);
    Scanner attrFil = new Scanner(fil);

    while (attrFil.hasNextLine()) {
      String nvn = attrFil.nextLine();
      String voksnebarn = attrFil.nextLine();
      boolean barn = (voksnebarn.equals("BARN"));
      int fra = Integer.parseInt (attrFil.nextLine());
      int til = Integer.parseInt (attrFil.nextLine());
      nyAttr (destNavn, nvn, barn, fra, til);
    }
  }

  public String finnBesteDest (boolean barn, int fra, int til) {
    int maxTreff = 0;
    String besteHittil = "";

    for (Destinasjon d : destKatalog.values()) {
      int antTreff = d.antallAktuelleAttr(barn, fra, til);
      if (antTreff > maxTreff) {
        maxTreff = antTreff;
        besteHittil = d.hentDestNavn();
      }
    }

    return besteHittil;
  }
}
